"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateContentModelTypescriptCode = exports.parseContentModelSchema = exports.ContentModelSchema = exports.ContentModelType = exports.ContentModelType_Object = exports.ContentModelObjectField = exports.NonObjectContentModelType = exports.ContentModelType_Number = exports.ContentModelType_Boolean = exports.ContentModelType_Datetime = exports.ContentModelType_String = exports.ContentModelTypeFieldProps = exports.ContentModelNumberValidation = exports.ContentModelStringValidation = exports.ContentModelEnumValue = exports.ContentModelDatetimeKind = exports.ContentModelBooleanKind = exports.ContentModelNumberKind = exports.ContentModelStringKind = void 0;
var ss = __importStar(require("superstruct"));
exports.ContentModelStringKind = ss.enums([
    "brand_id",
    "category_id",
    "color",
    "content_block_id",
    "dropdown",
    "file_id",
    "image_file_id",
    "product_id",
    "radio",
    "string",
    "textarea",
    "variant_id",
    "web_url",
]);
exports.ContentModelNumberKind = ss.enums(["number", "percentage", "rating"]);
exports.ContentModelBooleanKind = ss.enums(["checkbox", "toggle"]);
exports.ContentModelDatetimeKind = ss.enums(["date", "datetime"]);
exports.ContentModelEnumValue = ss.type({
    label: ss.string(),
    value: ss.string(),
});
exports.ContentModelStringValidation = ss.type({
    minLength: ss.optional(ss.number()),
    minLengthError: ss.optional(ss.string()),
    maxLength: ss.optional(ss.number()),
    maxLengthError: ss.optional(ss.string()),
    pattern: ss.optional(ss.string()),
    patternError: ss.optional(ss.string()),
    enum: ss.optional(ss.array(exports.ContentModelEnumValue)),
    defaultValue: ss.optional(ss.string()),
});
exports.ContentModelNumberValidation = ss.type({
    min: ss.optional(ss.number()),
    minError: ss.optional(ss.string()),
    max: ss.optional(ss.number()),
    maxError: ss.optional(ss.string()),
    integer: ss.optional(ss.boolean()),
});
exports.ContentModelTypeFieldProps = ss.type({
    name: ss.string(),
    label: ss.optional(ss.string()),
});
exports.ContentModelType_String = ss.type({
    type: ss.literal("string"),
    kind: exports.ContentModelStringKind,
    validation: ss.optional(exports.ContentModelStringValidation),
    help: ss.optional(ss.string()),
});
exports.ContentModelType_Datetime = ss.type({
    type: ss.literal("datetime"),
    kind: exports.ContentModelDatetimeKind,
    help: ss.optional(ss.string()),
});
exports.ContentModelType_Boolean = ss.type({
    type: ss.literal("boolean"),
    kind: exports.ContentModelBooleanKind,
    help: ss.optional(ss.string()),
});
exports.ContentModelType_Number = ss.type({
    type: ss.literal("number"),
    kind: exports.ContentModelNumberKind,
    validation: ss.optional(exports.ContentModelNumberValidation),
    help: ss.optional(ss.string()),
});
exports.NonObjectContentModelType = ss.union([
    exports.ContentModelType_String,
    exports.ContentModelType_Datetime,
    exports.ContentModelType_Boolean,
    exports.ContentModelType_Number,
]);
exports.ContentModelObjectField = ss.intersection([exports.NonObjectContentModelType, exports.ContentModelTypeFieldProps]);
exports.ContentModelType_Object = ss.type({
    type: ss.literal("object"),
    fields: ss.array(exports.ContentModelObjectField),
    help: ss.optional(ss.string()),
});
exports.ContentModelType = ss.union([exports.ContentModelType_Object, exports.NonObjectContentModelType]);
exports.ContentModelSchema = ss.type({
    label: ss.optional(ss.string()),
    name: ss.string(),
    json: exports.ContentModelType,
});
function parseContentModelSchema(data) {
    var j;
    try {
        j = JSON.parse(data);
    }
    catch (err) {
        throw new Error("failed parsing content model schema: invalid json");
    }
    var r = ss.validate(j, exports.ContentModelSchema);
    if (!r[0]) {
        return r[1];
    }
    else {
        var additionalInfo = r[0]
            .failures()
            .map(function (f) { return "  ".concat(f.message); })
            .join("\n");
        throw new Error("failed parsing content model schema: ".concat(r[0].message, "\ndetailed report:\n").concat(additionalInfo));
    }
}
exports.parseContentModelSchema = parseContentModelSchema;
function generateTypeFor(v) {
    var _a;
    switch (v.type) {
        case "boolean":
            return "ss.boolean()";
        case "string":
            var e = (_a = v.validation) === null || _a === void 0 ? void 0 : _a.enum;
            if (e) {
                return "ss.enums([".concat(e.map(function (v) { return "".concat(JSON.stringify(v.value), " as const"); }).join(","), "])");
            }
        case "datetime":
            return "ss.string()";
        case "number":
            return "ss.number()";
        case "object":
            var fields = [];
            for (var _i = 0, _b = v.fields; _i < _b.length; _i++) {
                var f = _b[_i];
                fields.push("".concat(JSON.stringify(f.name), ": ").concat(generateTypeFor(f)));
            }
            return "ss.type({".concat(fields.join(","), "})");
    }
}
function generateContentModelTypescriptCode(schemas) {
    var out = "";
    out += "import * as ss from \"superstruct\";\n";
    out += "export const schemas = {\n";
    for (var _i = 0, schemas_1 = schemas; _i < schemas_1.length; _i++) {
        var schema = schemas_1[_i];
        out += "".concat(JSON.stringify(schema.name), ": ").concat(generateTypeFor(schema.json), ",\n");
    }
    out += "};\n";
    return out;
}
exports.generateContentModelTypescriptCode = generateContentModelTypescriptCode;
