"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureOperationNamesAreUnique = exports.resolveTypes = exports.recursiveCopyFragments = exports.extractFragmentDeps = exports.addFragmentDepsRecursive = exports.extractNamedTypes = exports.walkTSType = exports.resolveTypesSorted = exports.sortDictValues = exports.convertType = exports.typeToString = exports.enumToString = exports.validateDocument = exports.logErrorMessage = exports.extractImportSpecs = exports.loadImports = exports.loadGraphQLFromFile = void 0;
var utilities_1 = require("graphql/utilities");
var graphql_1 = require("graphql");
var lodash_1 = __importDefault(require("lodash"));
var glob_1 = __importDefault(require("glob"));
var path_1 = __importDefault(require("path"));
var fs_1 = __importDefault(require("fs"));
var util_1 = require("./util");
function loadGraphQLFromFile(path) {
    var data = fs_1.default.readFileSync(path, "utf-8");
    return new graphql_1.Source(data, path);
}
exports.loadGraphQLFromFile = loadGraphQLFromFile;
function loadImports(specs, includes, schema, embedImports) {
    var uniqueDirs = lodash_1.default.uniq(lodash_1.default.map(specs, function (s) { return s.from; }));
    var importedData = {};
    var fragmentWhitelistPerDir = {};
    var prefixMap = {};
    var includeAbbrevs = lodash_1.default.compact(lodash_1.default.map(includes, function (inc) {
        var _a = lodash_1.default.split(inc, "=", 3), keyName = _a[0], val = _a[1], prefix = _a[2];
        if (keyName && val && prefix) {
            var key = "@".concat(keyName);
            prefixMap[key] = prefix;
            return { key: key, val: val };
        }
        else {
            return undefined;
        }
    }));
    if (embedImports) {
        // when embedding imports, collect fragment whitelists for each dir
        for (var _i = 0, uniqueDirs_1 = uniqueDirs; _i < uniqueDirs_1.length; _i++) {
            var d = uniqueDirs_1[_i];
            for (var _a = 0, specs_1 = specs; _a < specs_1.length; _a++) {
                var s = specs_1[_a];
                if (s.from !== d)
                    continue;
                if (s.what.kind === "all") {
                    fragmentWhitelistPerDir[d] = "all";
                    break;
                }
                else {
                    var dd = fragmentWhitelistPerDir[d];
                    if (!dd || lodash_1.default.isString(dd))
                        dd = {};
                    for (var _c = 0, _d = s.what.names; _c < _d.length; _c++) {
                        var name_1 = _d[_c];
                        dd[name_1] = true;
                    }
                    fragmentWhitelistPerDir[d] = dd;
                }
            }
        }
    }
    var rawImportData = [];
    for (var _e = 0, uniqueDirs_2 = uniqueDirs; _e < uniqueDirs_2.length; _e++) {
        var d = uniqueDirs_2[_e];
        var foundSome = false;
        var dd = d;
        for (var _f = 0, includeAbbrevs_1 = includeAbbrevs; _f < includeAbbrevs_1.length; _f++) {
            var ia = includeAbbrevs_1[_f];
            dd = dd.replace(ia.key, ia.val);
        }
        var files = glob_1.default.sync(path_1.default.join(dd, "**", "*.graphql"));
        if (files.length > 0) {
            // found some files in some dir, let's consider it a match
            var sources = [];
            for (var _g = 0, files_1 = files; _g < files_1.length; _g++) {
                var file = files_1[_g];
                var src = loadGraphQLFromFile(file);
                var imports = extractImportSpecs(src.body);
                if (imports.length > 0)
                    throw new Error("importing file ".concat(JSON.stringify(file), " which itself has imports, nested imports are not allowed"));
                if (src)
                    sources.push(src);
            }
            var doc = (0, utilities_1.concatAST)(lodash_1.default.map(sources, function (src) { return (0, graphql_1.parse)(src); }));
            var data = resolveTypesSorted(schema, doc, undefined);
            if (embedImports) {
                // when embedding imports, we only need those fragments which were explicitly requested and their deps
                // let's remove unused things here
                data.operations = {};
                data.operationsSorted = [];
                var whitelist = fragmentWhitelistPerDir[d];
                if (whitelist && whitelist !== "all") {
                    var newUsedNamedTypes = {};
                    var newFragments = {};
                    for (var key in whitelist) {
                        var visited = {};
                        recursiveCopyFragments(schema, newUsedNamedTypes, newFragments, data.fragments, visited, key, data.fragmentDeps);
                    }
                    data.fragments = newFragments;
                    data.fragmentsSorted = sortDictValues(newFragments);
                    data.usedNamedTypes = newUsedNamedTypes;
                    data.usedNamedTypesSorted = sortDictValues(newUsedNamedTypes);
                }
            }
            rawImportData.push({
                fragmentDeps: data.fragmentDeps,
                fragments: data.fragments,
                usedNamedTypes: data.usedNamedTypes,
            });
            importedData[d] = data;
            foundSome = true;
            break;
        }
        if (!foundSome) {
            throw new Error("failed resolving ".concat(JSON.stringify(d), " import path"));
        }
    }
    // all import files are loaded and resolved, let's sort out imports
    var importedFragments = {};
    for (var _h = 0, specs_2 = specs; _h < specs_2.length; _h++) {
        var s = specs_2[_h];
        var d = importedFragments[s.from] || (importedFragments[s.from] = {});
        var data = (0, util_1.nonNull)(importedData[s.from], "imported data");
        if (s.what.kind === "all") {
            lodash_1.default.extend(d, lodash_1.default.fromPairs(lodash_1.default.map(data.fragmentsSorted, function (f) { return [f.name, true]; })));
        }
        else if (s.what.kind === "some") {
            for (var _j = 0, _k = s.what.names; _j < _k.length; _j++) {
                var name_2 = _k[_j];
                var f = data.fragments[name_2];
                if (!f)
                    throw new Error("fragment ".concat(JSON.stringify(name_2), " not found in ").concat(JSON.stringify(s.from)));
                d[name_2] = true;
            }
        }
    }
    var outMap = {};
    lodash_1.default.forEach(importedFragments, function (d, path) {
        var data = (0, util_1.nonNull)(importedData[path], "imported data");
        lodash_1.default.forEach(d, function (_b, name) {
            var alreadyImported = outMap[name];
            if (alreadyImported) {
                var argName = JSON.stringify(name);
                var argPath = JSON.stringify(path);
                var argSecondPath = JSON.stringify(alreadyImported.path);
                throw new Error("fragment ".concat(argName, " import from ").concat(argPath, " and from ").concat(argSecondPath));
            }
            var fragment = (0, util_1.nonNull)(data.fragments[name], "fragment: ".concat(name));
            outMap[name] = {
                name: name,
                path: path,
                fragment: fragment,
                context: data,
            };
        });
    });
    return {
        loadedImports: lodash_1.default.sortBy(lodash_1.default.compact(lodash_1.default.values(outMap)), function (v) { return v.name; }),
        loadedImportsMap: outMap,
        rawImportData: rawImportData,
        embedImports: embedImports,
        prefixMap: prefixMap,
    };
}
exports.loadImports = loadImports;
function extractImportSpecs(data) {
    var out = [];
    var importAllFromRegexp = /import\s+\*\s+from\s+"([^"]+)"/g;
    var importSomeFromRegexp = /import\s+\{\s*([a-zA-Z0-9_]+)((?:\s*,\s*[a-zA-Z0-9_]+)*)\s*\}\s+from\s+"([^"]+)"/g;
    while (true) {
        var m = importAllFromRegexp.exec(data);
        if (!m) {
            break;
        }
        out.push({
            from: m[1],
            what: { kind: "all" },
        });
    }
    while (true) {
        var m = importSomeFromRegexp.exec(data);
        if (!m) {
            break;
        }
        var names = __spreadArray([m[1]], lodash_1.default.compact(lodash_1.default.map(lodash_1.default.split(m[2], ","), lodash_1.default.trim)), true);
        out.push({
            from: m[3],
            what: { kind: "some", names: names },
        });
    }
    return out;
}
exports.extractImportSpecs = extractImportSpecs;
function NoAnonymousQueries(context) {
    return {
        OperationDefinition: function (node) {
            if (!node.name) {
                context.reportError(new graphql_1.GraphQLError("Script does not support anonymous operations", [node]));
            }
            return false;
        },
    };
}
function logGraphQLError(error) {
    var fileName = error.source ? error.source.name : "<unknown>";
    if (error.locations) {
        for (var _i = 0, _a = error.locations; _i < _a.length; _i++) {
            var location_1 = _a[_i];
            logErrorMessage(error.message, fileName, location_1.line);
        }
    }
}
function logErrorMessage(message, fileName, lineNumber) {
    console.error("".concat(fileName, ":").concat(lineNumber, ": ").concat(message));
}
exports.logErrorMessage = logErrorMessage;
function validateDocument(schema, document, allowUnusedFragments) {
    var specifiedRulesToBeRemoved = [graphql_1.UniqueOperationNamesRule, graphql_1.KnownDirectivesRule];
    if (allowUnusedFragments) {
        specifiedRulesToBeRemoved.push(graphql_1.NoUnusedFragmentsRule);
    }
    var rules = __spreadArray([
        NoAnonymousQueries
    ], lodash_1.default.filter(graphql_1.specifiedRules, function (rule) { return !lodash_1.default.includes(specifiedRulesToBeRemoved, rule); }), true);
    var validationErrors = (0, graphql_1.validate)(schema, document, rules);
    if (validationErrors.length > 0) {
        for (var _i = 0, validationErrors_1 = validationErrors; _i < validationErrors_1.length; _i++) {
            var error = validationErrors_1[_i];
            logGraphQLError(error);
        }
        throw new Error("Validation of GraphQL query document failed");
    }
}
exports.validateDocument = validateDocument;
function walkType(t, ss, cb) {
    cb(t);
    if ((0, graphql_1.isNonNullType)(t) || (0, graphql_1.isListType)(t)) {
        walkType(t.ofType, ss, cb);
        return;
    }
    if ((0, graphql_1.isInputObjectType)(t)) {
        var fields = t.getFields();
        lodash_1.default.forEach(fields, function (f) {
            walkType(f.type, undefined, cb);
        });
        return;
    }
    if ((0, graphql_1.isObjectType)(t)) {
        if (!ss)
            throw new Error("should not be undefined");
        var fieldTypes = t.getFields();
        for (var _i = 0, _a = ss.selections; _i < _a.length; _i++) {
            var sel = _a[_i];
            switch (sel.kind) {
                case graphql_1.Kind.FIELD:
                    walkType(fieldTypes[sel.name.value].type, sel.selectionSet, cb);
                    break;
            }
        }
        return;
    }
}
function walkTypeNode(t, cb) {
    cb(t);
    if (t.kind === graphql_1.Kind.LIST_TYPE || t.kind === graphql_1.Kind.NON_NULL_TYPE) {
        walkTypeNode(t.type, cb);
    }
}
function identString(ident) {
    var result = "";
    for (var i = 0; i < ident; i++) {
        result += "  ";
    }
    return result;
}
function enumToString(t, ident) {
    if (ident === void 0) { ident = 0; }
    var result = "";
    var values = lodash_1.default.map(t.getValues(), function (v) { return v.name; }).sort();
    for (var i = 0; i < values.length; i++) {
        result += "\n".concat(identString(ident + 1), "\"").concat(values[i], "\"");
        if (i != values.length - 1) {
            result += " |";
        }
    }
    return result;
}
exports.enumToString = enumToString;
function typeToString(t, ident, useUndefined) {
    if (ident === void 0) { ident = 0; }
    if (useUndefined === void 0) { useUndefined = false; }
    switch (t.type) {
        case "object":
            var result = "{";
            var fields = lodash_1.default.keys(t.fields).sort();
            if (fields.length !== 0)
                result += "\n";
            for (var i = 0; i < fields.length; i++) {
                var ft = t.fields[fields[i]];
                if (!ft)
                    continue;
                result += "".concat(identString(ident + 1)).concat(fields[i]).concat(ft.nullable && useUndefined ? "?" : "", ": ");
                result += "".concat(typeToString(ft, ident + 1, useUndefined), ";\n");
            }
            result += "".concat(identString(ident), "}");
            return result + (t.nullable ? " | null" : "");
        case "named":
            return t.name + (t.nullable ? " | null" : "");
        case "array":
            return "Array<".concat(typeToString(t.element, ident, useUndefined), ">") + (t.nullable ? " | null" : "");
        case "intersection":
            var tstrs = lodash_1.default.map(t.types, function (t) { return typeToString(t, ident, useUndefined); });
            return "(".concat(lodash_1.default.join(tstrs, " & "), ")") + (t.nullable ? " | null" : "");
    }
}
exports.typeToString = typeToString;
function convertTypeNode(t, nullable) {
    if (nullable === void 0) { nullable = true; }
    switch (t.kind) {
        case graphql_1.Kind.LIST_TYPE:
            return { type: "array", element: convertTypeNode(t.type), nullable: nullable };
        case graphql_1.Kind.NAMED_TYPE:
            var name_3 = builtInScalarMap[t.name.value];
            return { type: "named", name: name_3 || t.name.value, nullable: nullable };
        case graphql_1.Kind.NON_NULL_TYPE:
            return convertTypeNode(t.type, false);
    }
}
function aliasOrName(v) {
    return v.alias ? v.alias.value : v.name.value;
}
var builtInScalarMap = (_a = {},
    _a[graphql_1.GraphQLString.name] = "string",
    _a[graphql_1.GraphQLInt.name] = "number",
    _a[graphql_1.GraphQLFloat.name] = "number",
    _a[graphql_1.GraphQLBoolean.name] = "boolean",
    _a[graphql_1.GraphQLID.name] = "string",
    _a);
function convertType(t, ss, fragments, imports, nullable, expandNamedInput) {
    var _a;
    if (nullable === void 0) { nullable = true; }
    if (expandNamedInput === void 0) { expandNamedInput = false; }
    if ((0, graphql_1.isNonNullType)(t))
        return convertType(t.ofType, ss, fragments, imports, false);
    if ((0, graphql_1.isEnumType)(t))
        return { type: "named", name: t.name, nullable: nullable };
    if ((0, graphql_1.isScalarType)(t)) {
        var name_4 = builtInScalarMap[t.name];
        if (!name_4)
            return { type: "named", name: "ArbitraryObjectType", nullable: nullable };
        return { type: "named", name: name_4, nullable: nullable };
    }
    if ((0, graphql_1.isListType)(t)) {
        var element = convertType(t.ofType, ss, fragments, imports);
        return { type: "array", element: element, nullable: nullable };
    }
    if ((0, graphql_1.isInputObjectType)(t)) {
        if (!expandNamedInput)
            return { type: "named", name: t.name, nullable: nullable };
        var fields_1 = {};
        var inputFields = t.getFields();
        lodash_1.default.forEach(inputFields, function (f) {
            fields_1[f.name] = convertType(f.type, undefined, fragments, imports, true, false);
        });
        return { type: "object", fields: fields_1, nullable: nullable };
    }
    if ((0, graphql_1.isObjectType)(t)) {
        // common case, if there is only one selection and it's a fragment spread,
        // use fragment named type
        if (!ss)
            throw new Error("should not be undefined");
        if (ss.selections.length === 1) {
            var sel = ss.selections[0];
            if (sel.kind === graphql_1.Kind.FRAGMENT_SPREAD) {
                return { type: "named", name: "".concat(sel.name.value, "Fragment"), nullable: nullable };
            }
        }
        var obj = { type: "object", fields: {}, nullable: nullable };
        var out = obj;
        // otherwise convert things normally
        var fieldTypes = t.getFields();
        for (var _i = 0, _c = ss.selections; _i < _c.length; _i++) {
            var sel = _c[_i];
            switch (sel.kind) {
                case graphql_1.Kind.FIELD:
                    var name_5 = aliasOrName(sel);
                    obj.fields[name_5] = convertType(fieldTypes[sel.name.value].type, sel.selectionSet, fragments, imports);
                    break;
                case graphql_1.Kind.FRAGMENT_SPREAD:
                    var fragment = fragments[sel.name.value] || ((_a = imports[sel.name.value]) === null || _a === void 0 ? void 0 : _a.fragment);
                    if (fragment && fragment.type.type === "object") {
                        // once we reach the point where we use intersection, remove nullable flag from its members
                        // intersection itself becomes nullable and that's enough
                        obj.nullable = undefined;
                        var isect = out.type === "intersection" ? out : { type: "intersection", types: [obj], nullable: nullable };
                        isect.types.splice(0, 0, { type: "named", name: "".concat(sel.name.value, "Fragment") });
                        out = isect;
                    }
                    else {
                        // what do we do here?
                        var reason = !fragment ? "non-existent fragment" : "invalid type: ".concat(fragment.type);
                        throw new Error("invalid fragment reference: ".concat(sel.name.value, " (").concat(reason, ")"));
                    }
                    break;
                case graphql_1.Kind.INLINE_FRAGMENT:
                    // TODO: embed it into fields
                    throw new Error("not implemented yet");
            }
        }
        return out;
    }
    return { type: "named", name: "<unknown>" };
}
exports.convertType = convertType;
function sortDictValues(d) {
    var out = [];
    for (var _i = 0, _a = lodash_1.default.keys(d).sort(); _i < _a.length; _i++) {
        var k = _a[_i];
        out.push({
            name: k,
            value: (0, util_1.nonNull)(d[k]),
        });
    }
    return out;
}
exports.sortDictValues = sortDictValues;
function resolveTypesSorted(schema, doc, imports) {
    var vals = resolveTypes(schema, doc, imports);
    var usedNamedTypes = vals.usedNamedTypes, fragments = vals.fragments, operations = vals.operations;
    return __assign({ usedNamedTypesSorted: sortDictValues(usedNamedTypes), fragmentsSorted: sortDictValues(fragments), operationsSorted: sortDictValues(operations) }, vals);
}
exports.resolveTypesSorted = resolveTypesSorted;
function walkTSType(t, cb) {
    switch (t.type) {
        case "named":
            return cb(t);
        case "array":
            if (!cb(t))
                return;
            walkTSType(t.element, cb);
            break;
        case "object":
            if (!cb(t))
                return;
            for (var _i = 0, _a = lodash_1.default.keys(t.fields).sort(); _i < _a.length; _i++) {
                var key = _a[_i];
                var f = (0, util_1.nonNull)(t.fields[key]);
                walkTSType(f, cb);
            }
            break;
    }
}
exports.walkTSType = walkTSType;
function extractNamedTypes(t, namedTypes) {
    walkTSType(t, function (t) {
        if (t.type === "object" && t.asNamed) {
            var name_6 = t.asNamed;
            namedTypes[name_6] = __assign({}, t);
            delete t["fields"];
            delete t["asNamed"];
            t.type = "named";
            t.name = name_6;
            return false;
        }
        else {
            return true;
        }
    });
}
exports.extractNamedTypes = extractNamedTypes;
function addFragmentDepsRecursive(deps, frags, fragsMap, imports, path) {
    if (path === void 0) { path = ".."; }
    var addKeyOrContinue = function (key) {
        if (deps[key])
            return true;
        deps[key] = true;
        return false;
    };
    for (var _i = 0, frags_1 = frags; _i < frags_1.length; _i++) {
        var frag = frags_1[_i];
        var moreDeps = fragsMap[frag];
        if (moreDeps) {
            if (addKeyOrContinue("".concat(path, ":").concat(frag)))
                continue;
            addFragmentDepsRecursive(deps, moreDeps, fragsMap, imports, path);
        }
        else {
            var imp = imports[frag];
            if (imp) {
                if (addKeyOrContinue("".concat(imp.path, ":").concat(frag)))
                    continue;
                addFragmentDepsRecursive(deps, imp.context.fragmentDeps[frag] || [], imp.context.fragmentDeps, imports, imp.path);
            }
            else {
                if (addKeyOrContinue("".concat(path, ":").concat(frag)))
                    continue;
                addFragmentDepsRecursive(deps, [], fragsMap, imports, path);
            }
        }
    }
}
exports.addFragmentDepsRecursive = addFragmentDepsRecursive;
function extractFragmentDeps(ss, deps) {
    if (!ss) {
        return;
    }
    for (var _i = 0, _a = ss.selections; _i < _a.length; _i++) {
        var sel = _a[_i];
        switch (sel.kind) {
            case graphql_1.Kind.FIELD:
                extractFragmentDeps(sel.selectionSet, deps);
                break;
            case graphql_1.Kind.FRAGMENT_SPREAD:
                deps[sel.name.value] = true;
                break;
            case graphql_1.Kind.INLINE_FRAGMENT:
                throw new Error("not implemented yet");
        }
    }
}
exports.extractFragmentDeps = extractFragmentDeps;
function recursiveCopyFragments(schema, usedNamedTypes, dst, src, visited, key, deps) {
    if (visited[key])
        return;
    visited[key] = true;
    var f = src[key];
    if (!f)
        return;
    dst[key] = f;
    var t = schema.getType(f.node.typeCondition.name.value);
    if (t) {
        walkType(t, f.node.selectionSet, function (t) {
            if ((0, graphql_1.isEnumType)(t))
                usedNamedTypes[t.name] = true;
        });
    }
    for (var _i = 0, _a = deps[key] || []; _i < _a.length; _i++) {
        var dep = _a[_i];
        recursiveCopyFragments(schema, usedNamedTypes, dst, src, visited, dep, deps);
    }
}
exports.recursiveCopyFragments = recursiveCopyFragments;
function resolveTypes(schema, doc, imports) {
    var usedNamedTypes = {};
    var fragmentDeps = {};
    var fragments = {};
    var operations = {};
    var nameDedup = {};
    var namedDefinitions = [];
    for (var _i = 0, _a = doc.definitions; _i < _a.length; _i++) {
        var def = _a[_i];
        if (def.kind === graphql_1.Kind.OPERATION_DEFINITION || def.kind === graphql_1.Kind.FRAGMENT_DEFINITION) {
            var name_7 = def.name.value;
            if (nameDedup[name_7])
                throw new Error("definition \"".concat(name_7, "\" (").concat(def.kind, ") already exists"));
            nameDedup[name_7] = true;
            namedDefinitions.push({ name: name_7, def: def, resolved: false });
        }
    }
    var importedFragments = (imports === null || imports === void 0 ? void 0 : imports.loadedImportsMap) || {};
    if (imports && imports.embedImports) {
        importedFragments = {};
        for (var _c = 0, _d = imports.rawImportData; _c < _d.length; _c++) {
            var data = _d[_c];
            lodash_1.default.extend(usedNamedTypes, data.usedNamedTypes);
            lodash_1.default.extend(fragmentDeps, data.fragmentDeps);
            lodash_1.default.extend(fragments, data.fragments);
        }
    }
    while (true) {
        var numUnresolvedBefore = lodash_1.default.sumBy(namedDefinitions, function (d) { return (d.resolved ? 0 : 1); });
        var errors = [];
        var _loop_1 = function (ndef) {
            if (ndef.resolved)
                return "continue";
            var def = ndef.def;
            try {
                switch (def.kind) {
                    case graphql_1.Kind.OPERATION_DEFINITION:
                        // when fragment whitelist is given, we're only interested in fragments
                        var name_8 = "";
                        var type = void 0;
                        switch (def.operation) {
                            case graphql_1.OperationTypeNode.MUTATION:
                                type = schema.getMutationType();
                                name_8 = "".concat(def.name.value, "Mutation");
                                break;
                            case graphql_1.OperationTypeNode.QUERY:
                                type = schema.getQueryType();
                                name_8 = "".concat(def.name.value, "Query");
                                break;
                            case graphql_1.OperationTypeNode.SUBSCRIPTION:
                                type = schema.getSubscriptionType();
                                name_8 = "".concat(def.name.value, "Subscription");
                                break;
                        }
                        if (!type)
                            throw new Error("missing ".concat(def.operation, " type"));
                        walkType(type, def.selectionSet, function (t) {
                            if ((0, graphql_1.isEnumType)(t))
                                usedNamedTypes[t.name] = true;
                        });
                        var varFields_1 = {};
                        lodash_1.default.forEach(def.variableDefinitions, function (vd) {
                            varFields_1[vd.variable.name.value] = convertTypeNode(vd.type);
                            walkTypeNode(vd.type, function (t) {
                                if (t.kind === graphql_1.Kind.NAMED_TYPE)
                                    usedNamedTypes[t.name.value] = true;
                            });
                        });
                        var variables = { type: "object", fields: varFields_1 };
                        var operation = convertType(type, def.selectionSet, fragments, importedFragments);
                        operation.nullable = false; // hack: top level types are not nullable
                        operations[name_8] = {
                            operation: operation,
                            variables: variables,
                            node: def,
                        };
                        break;
                    case graphql_1.Kind.FRAGMENT_DEFINITION:
                        // skipping this fragment?
                        var t = schema.getType(def.typeCondition.name.value);
                        if (t) {
                            walkType(t, def.selectionSet, function (t) {
                                if ((0, graphql_1.isEnumType)(t))
                                    usedNamedTypes[t.name] = true;
                            });
                            var type_1 = convertType(t, def.selectionSet, fragments, importedFragments);
                            type_1.nullable = false; // hack: top level types are not nullable
                            fragments[def.name.value] = { type: type_1, node: def };
                            var deps = {};
                            extractFragmentDeps(def.selectionSet, deps);
                            fragmentDeps[def.name.value] = lodash_1.default.keys(deps).sort();
                        }
                        break;
                }
                ndef.resolved = true;
            }
            catch (err) {
                if (def.loc) {
                    errors.push("".concat(def.loc.source.name, ":").concat(JSON.stringify(def.loc), " ").concat(err.toString()));
                }
                else {
                    throw err;
                }
                ndef.resolved = false;
            }
        };
        for (var _e = 0, namedDefinitions_1 = namedDefinitions; _e < namedDefinitions_1.length; _e++) {
            var ndef = namedDefinitions_1[_e];
            _loop_1(ndef);
        }
        var numUnresolvedAfter = lodash_1.default.sumBy(namedDefinitions, function (d) { return (d.resolved ? 0 : 1); });
        if (numUnresolvedBefore === numUnresolvedAfter && numUnresolvedBefore !== 0) {
            throw new Error("failed to resolve types: " + lodash_1.default.join(errors, "\n"));
        }
        if (numUnresolvedAfter === 0)
            break;
    }
    // additional pass on used named types to get their dependencies in as well
    for (var _f = 0, _g = lodash_1.default.keys(usedNamedTypes); _f < _g.length; _f++) {
        var name_9 = _g[_f];
        var t = schema.getType(name_9);
        if (t) {
            walkType(t, undefined, function (t) {
                if ((0, graphql_1.isInputObjectType)(t) || (0, graphql_1.isEnumType)(t)) {
                    usedNamedTypes[t.name] = true;
                }
            });
        }
    }
    return {
        usedNamedTypes: usedNamedTypes,
        operations: operations,
        fragments: fragments,
        fragmentDeps: fragmentDeps,
    };
}
exports.resolveTypes = resolveTypes;
function ensureOperationNamesAreUnique(doc) {
    var counter = 0;
    return (0, graphql_1.visit)(doc, {
        OperationDefinition: function (node) {
            return __assign(__assign({}, node), { name: __assign(__assign({}, node.name), { value: (node.name ? node.name.value : "") + "".concat(counter++) }) });
        },
    });
}
exports.ensureOperationNamesAreUnique = ensureOperationNamesAreUnique;
